@@ -1,9 +1,9 @@
-## dist.topo.R (2016-11-24)
+## dist.topo.R (2021-04-18)
 
 ##      Topological Distances, Tree Bipartitions,
 ##   Consensus Trees, and Bootstrapping Phylogenies
 
-## Copyright 2005-2016 Emmanuel Paradis, 2016 Klaus Schliep
+## Copyright 2005-2021 Emmanuel Paradis, 2016-2021 Klaus Schliep
 
 ## This file is part of the R-package `ape'.
 ## See the file ../COPYING for licensing issues.
@@ -17,47 +17,75 @@
 
 dist.topo <- function(x, y = NULL, method = "PH85")
 {
-    if (!is.null(y)) {
-        res <- .dist.topo(x, y, method = method)
-    } else {
-        n <- length(x) # number of trees
-        res <- numeric(n * (n - 1) /2)
-        nms <- names(x)
-        if (is.null(nms)) nms <- paste0("tree", 1:n)
+    method <- match.arg(method, c("PH85", "score"))
+    if (!is.null(y)) x <- c(x, y)
+    testroot <- any(is.rooted(x))
+    n <- length(x) # number of trees
+    res <- numeric(n * (n - 1) /2)
+    nms <- names(x)
+    if (is.null(nms)) nms <- paste0("tree", 1:n)
+
+    if (method == "PH85") {
+        if (testroot)
+            warning("Some trees were rooted: topological distances may be spurious.")
+
+        x <- .compressTipLabel(x)
+        ntip <- length(attr(x, "TipLabel"))
+        nnode <- sapply(x, Nnode)
+
+        foo <- function(phy, ntip) {
+            phy <- reorder(phy, "postorder")
+            pp <- bipartition2(phy$edge, ntip)
+            attr(pp, "labels") <- phy$tip.label
+            ans <- SHORTwise(pp)
+            sapply(ans, paste, collapse = "\r")
+        }
+
+        x <- lapply(x, foo, ntip = ntip)
+
+        k <- 0L
+        for (i in 1:(n - 1)) {
+            y <- x[[i]]
+            m1 <- nnode[i]
+            for (j in (i + 1):n) {
+                z <- x[[j]]
+                k <- k + 1L
+                res[k] <- m1 + nnode[j] - 2 * sum(z %in% y)
+            }
+        }
+    } else { # method == "score"
         k <- 0L
         for (i in 1:(n - 1)) {
             for (j in (i + 1):n) {
                 k <- k + 1L
-                res[k] <- .dist.topo(x[[i]], x[[j]], method = method)
+                ## still very slow......
+                res[k] <- .dist.topo.score(x[[i]], x[[j]])
             }
         }
-        attr(res, "Size") <- n
-        attr(res, "Labels") <- nms
-        attr(res, "Diag") <- attr(res, "Upper") <- FALSE
-        attr(res, "method") <- method
-        class(res) <- "dist"
     }
+
+    attr(res, "Size") <- n
+    attr(res, "Labels") <- nms
+    attr(res, "Diag") <- attr(res, "Upper") <- FALSE
+    attr(res, "method") <- method
+    class(res) <- "dist"
     res
 }
 
-.dist.topo <- function(x, y, method = "PH85")
+.dist.topo.score <- function(x, y)
 {
-    if (method == "PH85") {
-        bs <- bitsplits(unroot.multiPhylo(c(x, y)))
-        return(sum(bs$freq == 1))
-    }
-
-    ## method == "score":
     if (is.null(x$edge.length) || is.null(y$edge.length))
         stop("trees must have branch lengths for branch score distance.")
     nx <- length(x$tip.label)
-    x <- unroot(x)
-    y <- unroot(y)
-    bp1 <- .Call(bipartition, x$edge, nx, x$Nnode)
+    x <- reorder.phylo(unroot(x), "postorder")
+    y <- reorder.phylo(unroot(y), "postorder")
+    ##bp1 <- .Call(bipartition, x$edge, nx, x$Nnode)
+    bp1 <- bipartition2(x$edge, nx)
     bp1 <- lapply(bp1, function(xx) sort(x$tip.label[xx]))
     ny <- length(y$tip.label) # fix by Otto Cordero
     ## fix by Tim Wallstrom:
-    bp2.tmp <- .Call(bipartition, y$edge, ny, y$Nnode)
+    bp2.tmp <- bipartition2(y$edge, ny)
+    ##bp2.tmp <- .Call(bipartition, y$edge, ny, y$Nnode)
     bp2 <- lapply(bp2.tmp, function(xx) sort(y$tip.label[xx]))
     bp2.comp <- lapply(bp2.tmp, function(xx) setdiff(1:ny, xx))
     bp2.comp <- lapply(bp2.comp, function(xx) sort(y$tip.label[xx]))
@@ -126,11 +154,12 @@ prop.part <- function(..., check.labels = TRUE)
     class(obj) <- NULL # fix by Klaus Schliep (2014-03-06)
     for (i in 1:ntree) storage.mode(obj[[i]]$Nnode) <- "integer"
     class(obj) <- "multiPhylo"
-    obj <- .uncompressTipLabel(obj) # fix a bug (2010-11-18)
-    clades <- .Call(prop_part, obj, ntree, TRUE)
-    attr(clades, "number") <- attr(clades, "number")[1:length(clades)]
+    obj <- reorder(obj, "postorder")
+# the following line should not be necessary any more
+#    obj <- .uncompressTipLabel(obj) # fix a bug (2010-11-18)
+    nTips <- length(obj[[1]]$tip.label)
+    clades <- prop_part2(obj, nTips)
     attr(clades, "labels") <- obj[[1]]$tip.label
-    class(clades) <- "prop.part"
     clades
 }
 
@@ -163,7 +192,7 @@ plot.prop.part <- function(x, barcol = "blue", leftmar = 4, col = "red", ...)
     layout(matrix(1:2, 2, 1), heights = c(1, 3))
     par(mar = c(0.1, leftmar, 0.1, 0.1))
     one2L <- seq_len(L)
-    plot(one2L, attr(x, "number"), type = "h", col = barcol, xlim = c(0.5, L),
+    plot(one2L - 0.5, attr(x, "number"), type = "h", col = barcol, xlim = c(0, L),
          xaxs = "i", xlab = "", ylab = "Frequency", xaxt = "n", bty = "n", ...)
     M <- matrix(0L, L, n)
     for (i in one2L) M[i, x[[i]]] <- 1L
@@ -190,8 +219,8 @@ prop.clades <- function(phy, ..., part = NULL, rooted = FALSE)
     bp <- prop.part(phy)
     if (!rooted) {
         ## avoid messing up the order and length if phy is rooted in some cases
-        bp <- ONEwise(bp)
-        part <- postprocess.prop.part(part)
+        bp <- SHORTwise(bp)
+        part <- postprocess.prop.part(part, "SHORTwise")
     }
     pos <- match(bp, part)
     tmp <- which(!is.na(pos))
@@ -216,6 +245,11 @@ boot.phylo <-
     y <- nc <- ncol(x)
     nr <- nrow(x)
 
+    if (nr < 4 && !trees) {
+        warning("not enough rows in 'x' to compute bootstrap values.\nSet 'trees = TRUE' if you want to get the bootstrap trees")
+        return(integer())
+    }
+
     if (block > 1) {
         a <- seq(1, nc - 1, block)
         b <- seq(block, nc, block)
@@ -247,6 +281,9 @@ boot.phylo <-
         if (!quiet) cat(" done.")
     }
 
+    if (nr < 4 && trees)
+        return(list(BP = integer(), trees = boot.tree))
+
     if (!quiet) cat("\nCalculating bootstrap values...")
 
     ## sort labels after mixed them up
@@ -255,7 +292,7 @@ boot.phylo <-
         boot.tree <- .uncompressTipLabel(boot.tree)
         boot.tree <- unclass(boot.tree) # otherwise countBipartitions crashes
     }
-
+    class(boot.tree) <- "multiPhylo"
     if (rooted) {
         pp <- prop.part(boot.tree)
         ans <- prop.clades(phy, part = pp, rooted = rooted)
@@ -263,31 +300,32 @@ boot.phylo <-
         phy <- reorder(phy, "postorder")
         ints <- phy$edge[, 2] > Ntip(phy)
         ans <- countBipartitions(phy, boot.tree)
-        ans <- c(B, ans[order(phy$edge[ints, 2])])
+        ans <- c(NA_integer_, ans[order(phy$edge[ints, 2])])
     }
 
     if (!quiet) cat(" done.\n")
 
-    if (trees) {
-        class(boot.tree) <- "multiPhylo"
-        ans <- list(BP = ans, trees = boot.tree)
-    }
+    if (trees) ans <- list(BP = ans, trees = boot.tree)
     ans
 }
 
 ### The next function transforms an object of class "prop.part" so
-### that the vectors which are identical in terms of split are aggregated.
+### that the vectors which are identical in terms of splits are aggregated.
 ### For instance if n = 5 tips, 1:2 and 3:5 actually represent the same
 ### split though they are different clades. The aggregation is done
-### arbitrarily. The call to ONEwise() insures that all splits include
-### the first tip.
+### arbitrarily.
+### The call to SHORTwise() insures that all splits are the shortest ones.
+### The call to ONEwise() insures that all splits include the first tip.
 ### (rewritten by Klaus)
-postprocess.prop.part <- function(x)
+postprocess.prop.part <- function(x, method = "ONEwise")
 {
     w <- attr(x, "number")
     labels <- attr(x, "labels")
 
-    x <- ONEwise(x)
+    method <- match.arg(toupper(method), c("ONEWISE", "SHORTWISE"))
+    FUN <- switch(method, "ONEWISE" = ONEwise, "SHORTWISE" = SHORTwise)
+
+    x <- FUN(x)
     drop <- duplicated(x)
 
     if (any(drop)) {
@@ -309,14 +347,39 @@ postprocess.prop.part <- function(x)
 ### changed to 1:2.
 ONEwise <- function(x)
 {
-    v <- seq_along(x[[1L]])
-    for (i in 2:length(x)) {
+    nTips <- length(attr(x, "labels"))
+    v <- seq_len(nTips)
+    l <- lengths(x) == 0
+    if (any(l)) x[l] <- list(v)
+    for (i in which(!l)) {
         y <- x[[i]]
         if (y[1] != 1) x[[i]] <- v[-y]
     }
     x
 }
 
+### This function changes an object of class "prop.part" so that they
+### all include the shorter part of the partition.
+### For instance if n = 5 tips, 1:3 is changed to 4:5. In case n is even, e.g.
+### n = 6 similar to ONEwise.
+SHORTwise <- function(x) {
+    ## ensures the next line should also work for splits objects from phangorn
+    nTips <- length(attr(x, "labels"))
+    v <- seq_len(nTips)
+    l <- lengths(x)
+    lv <- nTips / 2
+    for (i in which(l >= lv)) {
+        y <- x[[i]]
+        if (l[i] > lv) {
+            x[[i]] <- v[-y]
+        } else { # (l[i] == lv) only possible alternative
+            if (y[1] != 1)
+                x[[i]] <- v[-y]
+        }
+    }
+    x
+}
+
 consensus <- function(..., p = 1, check.labels = TRUE)
 {
     foo <- function(ic, node) {
