@@ -1,20 +1,22 @@
-## multi2di.R (2010-01-23)
+## multi2di.R (2020-12-15)
 
-##   Collapse and Resolve Multichotomies
+##   Collapse or Resolve Multichotomies
 
-## Copyright 2005-2010 Emmanuel Paradis
+## Copyright 2005-2020 Emmanuel Paradis, 2018 Klaus Schliep
 
 ## This file is part of the R-package `ape'.
 ## See the file ../COPYING for licensing issues.
 
-multi2di <- function(phy, random = TRUE)
+multi2di <- function(phy, ...) UseMethod("multi2di")
+
+.multi2di_ape <- function(phy, random, equiprob, n)
 {
+    ## n: number of tips of phy
     degree <- tabulate(phy$edge[, 1])
     target <- which(degree > 2)
     if (!length(target)) return(phy)
     nb.edge <- dim(phy$edge)[1]
-    n <- length(phy$tip.label)
-    nextnode <- n + phy$Nnode + 1
+    nextnode <- n + phy$Nnode + 1L
     new.edge <- edge2delete <- NULL
     wbl <- FALSE
     if (!is.null(phy$edge.length)) {
@@ -32,9 +34,9 @@ multi2di <- function(phy, random = TRUE)
           ## so we store the result of sample()
             tmp <- sample(length(desc))
             desc <- desc[tmp]
-            res <- rtree(N)$edge
+            res <- if (equiprob) rtopology(N, rooted = TRUE)$edge else rtree(N)$edge
         } else {
-            res <- matrix(0, 2*N - 2, 2)
+            res <- matrix(0L, 2*N - 2, 2)
             res[, 1] <- N + rep(1:(N - 1), each = 2)
             res[, 2] <- N + rep(2:N, each = 2)
             res[seq(1, by = 2, length.out = N - 1), 2] <- 1:(N - 1)
@@ -49,7 +51,7 @@ multi2di <- function(phy, random = TRUE)
         ## now substitute the nodes in `res'
         ## `node' stays at the "root" of these new
         ## edges whereas their "tips" are `desc'
-        Nodes <- c(node, seq(from = nextnode, length.out = N - 2))
+        Nodes <- c(node, nextnode:(nextnode + N - 3L))
         res[, 1] <- Nodes[res[, 1] - N]
         tmp <- res[, 2] > N
         res[tmp, 2] <- Nodes[res[tmp, 2] - N]
@@ -57,8 +59,8 @@ multi2di <- function(phy, random = TRUE)
         new.edge <- rbind(new.edge, res)
         edge2delete <- c(edge2delete, ind)
         if (wbl) new.edge.length <- c(new.edge.length, el)
-        nextnode <- nextnode + N - 2
-        phy$Nnode <- phy$Nnode + N - 2
+        nextnode <- nextnode + N - 2L
+        phy$Nnode <- phy$Nnode + N - 2L
     }
     phy$edge <- rbind(phy$edge[-edge2delete, ], new.edge)
     if (wbl)
@@ -67,7 +69,7 @@ multi2di <- function(phy, random = TRUE)
     if (!is.null(phy$node.label))
         phy$node.label <-
             c(phy$node.label, rep("", phy$Nnode - length(phy$node.label)))
-    phy <- reorder(phy)
+    phy <- .reorder_ape(phy, "cladewise", FALSE, n, 1L) # fix by Klaus (2017-01-16)
 
     ## the node numbers are not in increasing order in edge[, 2]: this
     ## will confuse drop.tip and other functions (root), so renumber them
@@ -89,37 +91,69 @@ multi2di <- function(phy, random = TRUE)
     phy
 }
 
-di2multi <- function(phy, tol = 1e-8)
+multi2di.phylo <- function (phy, random = TRUE, equiprob = TRUE, ...)
+    .multi2di_ape(phy, random, equiprob = equiprob, length(phy$tip.label))
+
+multi2di.multiPhylo <- function(phy, random = TRUE, equiprob = TRUE, ...)
 {
-    if (is.null(phy$edge.length)) stop("the tree has no branch length")
-    ## We select only the internal branches which are
-    ## significantly small:
-    ind <- which(phy$edge.length < tol & phy$edge[, 2] > length(phy$tip.label))
-    n <- length(ind)
-    if (!n) return(phy)
-    ## recursive function to `propagate' node #'s in case
-    ## there is a series of consecutive edges to remove
-    foo <- function(ancestor, des2del) {
-        wh <- which(phy$edge[, 1] == des2del)
-        for (k in wh) {
-            if (phy$edge[k, 2] %in% node2del) foo(ancestor, phy$edge[k, 2])
-            else phy$edge[k, 1] <<- ancestor
-        }
+    labs <- attr(phy, "TipLabel")
+    oc <- oldClass(phy)
+    class(phy) <- NULL
+    if (is.null(labs)) phy <- lapply(phy, multi2di.phylo, random = random, equiprob = equiprob)
+    else {
+        phy <- lapply(phy, .multi2di_ape, random = random, equiprob = equiprob, n = length(labs))
+        attr(phy, "TipLabel") <- labs
     }
+    class(phy) <- oc
+    phy
+}
+
+di2multi <- function(phy, ...) UseMethod("di2multi")
+
+## by Klaus (2018-05-28)
+.di2multi_ape <- function (phy, tol = 1e-08, ntips)
+{
+    if (is.null(phy$edge.length)) stop("the tree has no branch length")
+    phy <- reorder(phy)
+    e1 <- seq_len(max(phy$edge))
+    ind <- which(phy$edge.length < tol & phy$edge[, 2] > ntips)
+    n <- length(ind)
+    if (!n) return(phy)
+
+    for (i in ind)
+        e1[phy$edge[i,2]] <-  e1[phy$edge[i,1]]
+
+    phy$edge[, 1] <- e1[phy$edge[, 1]]
     node2del <- phy$edge[ind, 2]
-    anc <- phy$edge[ind, 1]
-    for (i in 1:n) {
-        if (anc[i] %in% node2del) next
-        foo(anc[i], node2del[i])
-    }
     phy$edge <- phy$edge[-ind, ]
     phy$edge.length <- phy$edge.length[-ind]
+
     phy$Nnode <- phy$Nnode - n
-    ## Now we renumber the nodes that need to be:
-    sel <- phy$edge > min(node2del)
-    for (i in which(sel))
-      phy$edge[i] <- phy$edge[i] - sum(node2del < phy$edge[i])
+
+    e1 <- sort(unique(phy$edge[, 1]))
+    tmp <- integer(max(phy$edge))
+    tmp[e1] <- ntips + seq_len(phy$Nnode)
+    tmp[1:ntips] <- seq_len(ntips)
+
+    phy$edge[] <- tmp[phy$edge]
     if (!is.null(phy$node.label))
-        phy$node.label <- phy$node.label[-(node2del - length(phy$tip.label))]
+        phy$node.label <- phy$node.label[-(node2del - ntips)]
+    phy
+}
+
+di2multi.phylo <- function (phy, tol = 1e-08, ...)
+    .di2multi_ape(phy, tol, length(phy$tip.label))
+
+di2multi.multiPhylo <- function(phy, tol = 1e-08, ...)
+{
+    labs <- attr(phy, "TipLabel")
+    oc <- oldClass(phy)
+    class(phy) <- NULL
+    if (is.null(labs)) phy <- lapply(phy, di2multi.phylo, tol = tol)
+    else {
+        phy <- lapply(phy, .di2multi_ape, tol = tol, ntips = length(labs))
+        attr(phy, "TipLabel") <- labs
+    }
+    class(phy) <- oc
     phy
 }
